import Graphics.UI.GLUT
import Data.IORef

-- to get some more performance set antiAliasing to False
antiAliasing = True

-- helper functions
color3f = color :: Color3 GLfloat -> IO ()
translatef = translate :: Vector3 GLfloat -> IO ()
scalef = scale :: GLfloat -> GLfloat -> GLfloat -> IO ()
scale1 a = scalef a a a
rotatef = rotate :: GLfloat -> Vector3 GLfloat -> IO ()

-- display function, using time <- get elapsedTime
display time = do
    clear [ ColorBuffer ]
    loadIdentity
    time <- get elapsedTime
    let t = 0.001 * fromIntegral time
    mapM_ (\(x,y) -> do
      deploySpiral (t+x/10+y/10) x y (-8) 2
      ) [ (x,y) | x <- [-60,-40..60], y <- [-40,-20..40]]
    swapBuffers
    
-- deploySpiral with parameters: local time, x, y, start, end
deploySpiral t x y start end = do
    let k = (t-start)/(end-start)
    preservingMatrix $ do
      translatef (Vector3 x y 0)
      preservingMatrix $ do
	scale1 8
	scale1 ( sin ( pi*k )^2+1 )
	--scale1 ( ((sin(((-5*k+5)*2)^2)^2 )*(1-k)/2+0.5+k/2) )
	--rotate ( -(1-k)^6*10000 + t*180 ) $ Vector3 0 0 (1::GLfloat)
	rotate ( k*100 ) $ Vector3 0 0 (1::GLfloat)
	drawSpiral k

-- one creature
drawSpiral k = do
  mapM_ (\x -> preservingMatrix $ do
    rotatef x (Vector3 0 0 1)
    spiral k
    ) [-90,-75..90]

-- one line of a creature
spiral k = do
  renderPrimitive LineStrip $ do
    mapM_ (\(x,y) -> do
      color3f (Color3 (cos(y*k*pi)^2) (sin(x*k*pi)^2) (sin(y*k*pi)^2*0.3))
--       color3f (Color3 ((y+1)/1.17*(sin(k))^2) 0 (x/0.59))
      vertex $ Vertex3 x y 0.0
      ) spiralPoints

-- line generation x,y -> (sin(pi*k/l))*k/l , (cos(pi*k/l))*k/l with l segments
spiralPoints :: [(GLfloat,GLfloat)]
spiralPoints = map (\k -> ((sin (pi * k/l)) * k/l, (cos(pi * k/l)) * k/l)) [1..l]
  where l = 20
	
-- projection and resize handling
reshape :: ReshapeCallback
reshape size@(Size w h) = do
   viewport $= (Position 0 0, size)
   matrixMode $= Projection
   loadIdentity
   let wf = fromIntegral w
       hf = fromIntegral h
   if w <= h
      then ortho (-50) 50 (-50 * hf/wf) (50 * hf/wf) (-1) 1
      else ortho (-50 * wf/hf) (50 * wf/hf) (-50) 50 (-1) 1
   matrixMode $= Modelview 0

-- time is going on
idle time = do
    postRedisplay Nothing

-- OpenGL init and various stuff, idleCallback $= Just (idle time) for the time
main :: IO ()
main = do
  (progName,_) <- getArgsAndInitialize
  initialDisplayMode $= [DoubleBuffered]
  initialWindowSize $= Size 1024 768
  initialWindowPosition $= Position 100 100
  createWindow progName
  
  if antiAliasing == True then do
    lineSmooth $= Enabled
    blend $= Enabled
    blendFunc $= (SrcAlpha, OneMinusSrcAlpha)
    hint LineSmooth $= DontCare
    lineWidth $= 2
    else
    return ()

  time <- get elapsedTime
  idleCallback $= Just (idle time)
  displayCallback $= (display time)
  reshapeCallback $= Just reshape
  mainLoop